
###########################################################################################################
#
#  runrunrun
#  
#  @author simon jokuschies
#  @version 1.0
#  @contact info@leafpictures.de
#  @website www.leafpictures.de
#
#  description:
#  ever wanted to know who is faster? A blur or a roto? bet and check it out :-D
#  in the settings panel (scripts/runrunrun/settings) you can also edit
#  some values like speed, number of games and game width
#  You can check the global winning status of you and nuke if you click "scripts/runrunrun/show scores
#  have fun :-)  
#
#  instalation
#
#  put runrunrun folder in nuke home directory
#  in init.py write this line (without the '#' in the beginning):
#
#  nuke.pluginAddPath('runrunrun')
#
##########################################################################################################

#no need to change anything from here. Edit only if you exactly know what you're doing.

import nuke
import nukescripts
import threading
import thread
import time
import random
import math
import os
import webbrowser

global runsettingsdir
global runSettingsFile
global scoreFile
runSettingsDir=os.getenv("HOME")+"/.nuke/runrunrun"
runSettingsFile=runSettingsDir+"/settings.txt"
scoreFile=runSettingsDir+"/scores.txt"

def openFileReturnArr(file):
	'''
	open set file, read in all lines and
	return an array with all the lines
	'''

	arr=[]
	fobj = open("%s"%file, "r")
	#load in all lines
	for line in fobj:
		#delete word wrap at the end of each line
		line=line.replace("\n", "")
		arr.append(line)
	fobj.close()
	return arr

def getRunSetting(settingsFile, val):
	'''
	getValues by finding pattern in array.
	for getting the valuesout of the array which
	comes from the settings text file
	'''

	arr = openFileReturnArr(settingsFile)
	i=0
	for line in arr:
		findVal=arr[i].find("%s"%val)
		#if pattern found
		if findVal!=-1:
			val=arr[i]
			valArr=val.split("=")
			try:
				val=valArr[1] #value
				if val=="":
					val=" "
				elif val=="NONE":
					val=" "
			except:
				val="---"
		i+=1
	return val

def initRunSettings():
	'''
	initialize settings for runrunrun
	'''
	
	global speed
	global numberOfGames
	global gameWidth

	if not os.path.isdir(runSettingsDir):
		os.makedirs(runSettingsDir)
	if not os.path.isfile(runSettingsFile):
		rs = open(runSettingsFile,'w+')
		rs.write("@speed=60\n")
		rs.write("@numberOfGames=3\n")
		rs.write("@gameWidth=700\n")
		rs.close()		
	else:
		rs=open(runSettingsFile,'r+')
		rs.close()
	if not os.path.isfile(scoreFile):
		sf=open(scoreFile,'w+')
		sf.write("@you=0\n")
		sf.write("@nuke=0\n")
		sf.close()

	speed=int ( getRunSetting(runSettingsFile,"@speed") )
	numberOfGames=int ( getRunSetting(runSettingsFile,"@numberOfGames") )
	gameWidth=int ( getRunSetting(runSettingsFile,"@gameWidth") )

def moveNode(nodes):
	'''
	move nodes by random value
	'''

	global movement
	movement=7
	nodes["xpos"].setValue(nodes["xpos"].getValue()+ random.randrange(1,movement+1) )    

def builtGame():
	'''
	build all the nodes for game
	'''

	global gameNodes
	global backdrop
	gameNodes=[]

	player01 = nuke.createNode("Blur", inpanel=False)
	player01.setName("THE_BLUR_DUDE")
	player01.setInput(0,None)

	player02 = nuke.createNode("Roto", inpanel=False)
	player02.setName("THE_ROTO_DUDE")
	player02.setInput(0,None)
	player02.setXYpos( int(player01["xpos"].getValue()), int(player01["ypos"].getValue()+80) )

	start01 = nuke.createNode("Dot")
	start01.setXYpos( int(player01["xpos"].getValue())-10 , int(player01["ypos"].getValue())-100 )
	start01.setInput(0,None)
	start02 = nuke.createNode("Dot")
	start02.setXYpos( int(player01["xpos"].getValue())-10 , int(player02["ypos"].getValue())+140 )
	
	end01 = nuke.createNode("Dot")
	end01.setXYpos( int(player01["xpos"].getValue())+ int (gameWidth) , int(player01["ypos"].getValue())-100 )
	end01.setInput(0,None)
	end02 = nuke.createNode("Dot")
	end02.setXYpos( int(player01["xpos"].getValue())+ int (gameWidth) , int(player02["ypos"].getValue())+140 )

	flag = nuke.createNode("StickyNote", inpanel=False)
	flag["tile_color"].setValue(4294638335)
	flag["label"].setValue("<img src='flag.png' />")
	flag.setXYpos( int ( end01["xpos"].getValue() ) , int ( end01["ypos"].getValue() - 100 ) )
	flag.setSelected(False)

	backdrop = nuke.createNode("BackdropNode")
	backdrop["tile_color"].setValue(1414812927)
	backdrop.setXYpos ( int(start01["xpos"].getValue() - 50) , int (flag["ypos"].getValue() - 50) )
	backdrop["bdwidth"].setValue( int ( end01["xpos"].getValue() - start01["xpos"].getValue() + 200 ) )
	backdrop["bdheight"].setValue( int ( start02["ypos"].getValue() - flag["ypos"].getValue() + 100 ) )
	backdrop.setName("runrunrun")
	backdrop["note_font_size"].setValue(50)
	backdrop["label"].setValue("run nodes ruuuuuun: Round 1/%i" % numberOfGames)
	backdrop["note_font_color"].setValue(842150655)

	gameNodes.append(player01)
	gameNodes.append(player02)
	gameNodes.append(start01)
	gameNodes.append(start02)
	gameNodes.append(end01)
	gameNodes.append(end02)
	gameNodes.append(flag)
	gameNodes.append(backdrop)
	
	nuke.zoom( 1, [ player01.xpos()+350, player01.ypos()  ])
	
	return [player01,player02]

def pickNodePanel():
	'''
	panel to choose your favourite
	'''

	p=nuke.Panel("Select your best horse")
	p.setWidth(300)
	p.addEnumerationPulldown("Who will win?", "THE_BLUR_DUDE THE_ROTO_DUDE")
	return p

def settings():
	'''
	panel to choose your favourite
	'''

	initRunSettings()

	def buildPanel():
		p=nuke.Panel("runrunrun settings")
		p.setWidth(300)
		p.addSingleLineInput("speed", speed)
		p.addSingleLineInput("number of games", numberOfGames)
		p.addSingleLineInput("game width", gameWidth)
		return p
	
	p=buildPanel()
	if p.show():
		input_speed = p.value("speed") 
		input_numberOfGames = p.value("number of games") 
		input_gameWidth	= p.value("game width") 

		if isNumber(input_speed):
			if isNumber(input_numberOfGames):
				if isNumber(input_gameWidth):		
					rs = open(runSettingsFile,'w+')
					rs.write("@speed=%s\n" % input_speed)
					rs.write("@numberOfGames=%s\n" % input_numberOfGames)
					rs.write("@gameWidth=%s\n" % input_gameWidth)
					rs.close()		
				else:
					nuke.message("Please enter a NUMBER for game width")
					settings()
			else:
				nuke.message("Please enter a NUMBER for number of games")
				settings()
		else:
			nuke.message("Please enter a NUMBER for speed")
			settings()

def help():
	'''
	open webbrowser for help
	'''
	
	url = 'http://www.leafpictures.de/runrunrun'
	webbrowser.open_new(url)

def isNumber(input):
    '''
    check if the input only contains numbers
    '''
    
    try:
        int(input)
        return True
    except ValueError:
        return False

def showWinner(winner):
	'''
	display winner node
	'''

	global scoreYou
	global scoreNuke
	global scoreFile
	global matchWinner
	global backdrop
	global numberOfGames

	if winner==nodePicked:
		scoreYou+=1
	else:
		scoreNuke+=1

	if gamesPlayed!=numberOfGames:
		if winner==nodePicked:
			nuke.message("YOU WIN!\n\nyou: %i -- " % scoreYou + "nuke: %i" %scoreNuke )
		else:
			nuke.message("YOU LOSE!\n\nyou: %i -- " % scoreYou + "nuke: %i" %scoreNuke )

		lbl = "run nodes ruuuuuun: Round %i" % (gamesPlayed + 1) + "/%i" % numberOfGames
		backdrop["label"].setValue(lbl)
		
	else:
		if wins>float(numberOfGames)/2:
			nuke.message("YOU WIN THE MATCH!\n\n\nyou: %i -- " % scoreYou + "nuke: %i" %scoreNuke )
			matchWinner="you"
		elif wins<float(numberOfGames)/2:
			nuke.message("NUKE WINS THE MATCH!\n\n\nyou: %i -- " % scoreYou + "nuke: %i" %scoreNuke )
			matchWinner="nuke"
		elif wins==float(numberOfGames)/2:
			nuke.message("IT'S A DRAW!\n\nyou: %i -- " % scoreYou + "nuke: %i" %scoreNuke )

		nuke.executeInMainThread(writeMatchWinner, args=("%s" % matchWinner) )

		for n in gameNodes:
			nuke.delete(n)

def writeMatchWinner(matchWinner):
	'''
	write winner to scoreFile
	'''
	global scoreFile

	if not os.path.isfile(scoreFile):
		sf=open(scoreFile,'w+')
		sf.write("@you=0\n")
		sf.write("@nuke=0\n")
	else:
		sf=open(scoreFile,'r+')	
		
	scoreYou=int ( getRunSetting(scoreFile,"@you") )
	scoreNuke=int ( getRunSetting(scoreFile,"@nuke") )
	sf.close()

	sf=open(scoreFile,'w+')

	if matchWinner=="you":
		scoreYou+=1
	elif matchWinner=="nuke":
		scoreNuke+=1
	
	sf.write("@you=%i\n"%scoreYou)
	sf.write("@nuke=%i\n"%scoreNuke)

	sf.close()

def showScores():
	'''
	panel to show the scores
	'''
	
	if not os.path.isfile(scoreFile):
		sf=open(scoreFile,'w+')
		sf.write("@you=0\n")
		sf.write("@nuke=0\n")
	else:
		sf=open(scoreFile,'r+')	
		
	scoreYou=int ( getRunSetting(scoreFile,"@you") )
	scoreNuke=int ( getRunSetting(scoreFile,"@nuke") )
	sf.close()

	nuke.message("runrunrun scores\n\nYou: %i - " % scoreYou + "nuke: %i" % scoreNuke)

def runTh():
	'''
	running loop thread
	'''

	global gamesPlayed
	global wins

	gamesPlayed=0
	wins=0

	player1xStart=player[0]["xpos"].getValue()
	player2xStart=player[1]["xpos"].getValue()
	
	while gamesPlayed<numberOfGames:
		winner=""
		run=True

		while run==True:
			nuke.executeInMainThread( moveNode, args=( player[0] ) )
			nuke.executeInMainThread( moveNode, args=( player[1] ) )
	        
			if player[0]["xpos"].getValue() + player[0].screenWidth() > player1xStart + gameWidth:
				run=False
				winner=player[0].name()
			if player[1]["xpos"].getValue() + player[1].screenWidth() > player2xStart + gameWidth:
				run=False
				winner=player[1].name()
			time.sleep(1.0/speed)
		else:
			nuke.executeInMainThread(showWinner, args="%s" % winner )   
			if winner==nodePicked:
				wins+=1
			player[0]["xpos"].setValue(player1xStart)
			player[1]["xpos"].setValue(player2xStart)
			gamesPlayed+=1

def runrunrun():
	'''
	init game
	'''

	global nodePicked
	global scoreYou
	global scoreNuke
	global player

	scoreYou=0
	scoreNuke=0
	initRunSettings()
	player=builtGame()
	p=pickNodePanel()
	if p.show():
		nodePicked=p.value("Who will win?")
		threading.Thread( None, runTh ).start()
	else:
		for n in gameNodes:
			nuke.delete(n)
    
